<?php

/*
Plugin Name: Atomicon Gallery
Plugin URI: http://atomicon.nl/
Description: Atomicon Gallery for Wordpress
Version: 1.0
Author: Atomicon
Author URI: http://atomicon.nl/
License: GPLv2
*/

class atomicon_gallery_plugin
{
    private $_title = 'Atomicon Gallery';
    private $_slug = 'atomicon-gallery';
    private $_options = array();
    private $_messages = array();
    private $_attributes = array();

    /**
     * atomicon_gallery_plugin::__construct()
     *
     * @param string $title Title of the plugin
     * @return void
     */
    function __construct($title = 'Atomicon Gallery')
    {
        $this->_title = $title;
        $this->_slug = $this->slug($this->_title);
        $this->_options = get_option($this->slug(), array());
        $this->_messages = $this->get_session('messages', array('updated' => array(), 'error' => array(), ));

        $this->unset_session('messages');

        add_action('init', array($this, 'init'));

        $page = isset($_GET['page']) ? $_GET['page'] : '';

        $this->load_model('atomicon_gallery');

        $this->atomicon_gallery->init_images_folder();

        if (is_admin())
        {
            add_action('admin_menu', array($this, 'admin_menu'));
            if ($page == $this->slug())
            {
            	add_action('admin_head', array($this, 'head'));
                add_action('admin_print_scripts', array($this, 'print_scripts'));
                add_action('admin_print_styles', array($this, 'print_styles'));
                add_action('wp_ajax_atomicon_gallery_resize_file', array($this, 'wp_ajax_atomicon_gallery_resize_file'));
            }
        }
        else
        {
			add_shortcode($this->slug(), array($this, 'shortcode') );
            add_action('wp_head', array($this, 'head'));
            add_action('wp_print_scripts', array($this, 'print_scripts'));
            add_action('wp_print_styles', array($this, 'print_styles'));
        }
    }

	function shortcode($attributes, $content)
	{
		$html = '';

		$this->_attributes = is_array($attributes) ? $attributes : array();

		$folder  = isset($attributes['folder']) ? $attributes['folder'] : '';
		$gallery = $this->atomicon_gallery->get_gallery($folder);
		$class   = isset($attributes['class']) ? $attributes['class'] : '';
		$width   = isset($attributes['width']) ? (int)$attributes['width'] : 0;
		$height  = isset($attributes['height']) ? (int)$attributes['height'] : 0;
		$preview = isset($attributes['preview']) ? (int)$attributes['preview'] : 0;

		if ($preview>0)
		{
			if (is_single() || is_page())
			{
				return $html;
			}
		}

		$class = $class == '' ? 'atomicon-gallery-image' : $class;

		$data = array(
			'folder' => $folder,
			'gallery' => $gallery,
			'class' => $class,
			'width' => $width,
			'height' => $height,
			'preview' => $preview,
		);

		ob_start();

		$this->load_view('front-end', $data);

		$html = ob_get_contents();
		ob_end_clean();
		return $html;
	}

	/**
	 * atomicon_gallery_plugin::wp_ajax_atomicon_gallery_resize_file()
	 *
	 * @return void
	 */
	function wp_ajax_atomicon_gallery_resize_file()
	{
		$message = '';
		$ag_data = isset($_POST['ag_data']) ? $_POST['ag_data'] : '';
		parse_str($ag_data);

		if (isset($ag_file))
		{
			$images_folder = $this->atomicon_gallery->get_images_folder($ag_folder);

			if (isset($ag_thumbnails) && isset($ag_thumbnails['process']))
			{
    			$width = (int)$ag_thumbnails['width'];
    			$height = (int)$ag_thumbnails['height'];
    			if ($width>0 && $height>0)
    			{
	    			$orig = $images_folder . $ag_file;
	    			$dest = $images_folder . 'ag-tn-'.$ag_file;
	       			if ($this->atomicon_gallery->gd_resize($width, $height, $orig, $dest)===FALSE)
	       			{
	       				$message .= sprintf(__('Error generating thumbnail for %s'."\n"), $ag_file);
	       			}
     			}
			}


			if (isset($ag_images) && isset($ag_images['process']))
			{
    			$width = (int)$ag_images['width'];
    			$height = (int)$ag_images['height'];
    			if ($width>0 && $height>0)
    			{
	    			$orig = $images_folder . $ag_file;
	    			$dest = $images_folder . $ag_file;
	       			if ($this->atomicon_gallery->gd_resize($width, $height, $orig, $dest)===FALSE)
	       			{
	       				$message .= sprintf(__('Error resizing image %s'."\n"), $ag_file);
       				}
     			}
			}
		}
		echo $message;

		exit();
	}

    /**
     * atomicon_gallery_plugin::init()
     *
     * @return void
     */
    function init()
    {
        load_plugin_textdomain($this->slug(), false, dirname(plugin_basename(__FILE__)) . '/languages');
        $this->mce_add_buttons();
    }

    function mce_add_buttons()
	{
		// Don't bother doing this stuff if the current user lacks permissions
		if ( !current_user_can('edit_posts') && !current_user_can('edit_pages') )
		{
			return;
		}

		// Add only in Rich Editor mode
		if ( get_user_option('rich_editing') == 'true')
		{
			add_filter('mce_external_plugins', array($this, 'mce_external_plugins'));
			add_filter('mce_buttons', array($this, 'mce_buttons'), 0);
		}
	}

	function mce_buttons($buttons)
	{
		array_push($buttons, "separator", 'atomicongallery');
		return $buttons;
	}

	// Load the TinyMCE plugin : editor_plugin.js (wp2.5)
	function mce_external_plugins($plugin_array)
	{
		$plugin_array['atomicongallery'] = $this->plugins_url('atomicon_gallery_editor_plugin.js');
		return $plugin_array;
	}


    /* Standard wordpress actions */
    /**
     * atomicon_gallery_plugin::admin_menu()
     *
     * @return void
     */
    function admin_menu()
    {
        $screen = add_menu_page($this->_title, $this->_title, 'administrator', $this->slug(), array($this, 'admin_page'), $this->plugins_url('assets/images/icon-atomicon.png'));

        add_contextual_help($screen, $this->load_view('help', array(), true ) );
    }

    /**
     * atomicon_gallery_plugin::head()
     *
     * @return void
     */
    function head()
    {
        //Room for custom scripts and or stylesheets
        //echo '<link rel="stylesheet" href="'.$this->plugins_url('assets/css/style.css').'" type="text/css" />'."\n";
        //echo '<script src="'.$this->plugins_url('js/script.js').'"></script>'."\n";
        $this->load_view('head');

    }

    /**
     * atomicon_gallery_plugin::print_scripts()
     *
     * @return
     */
    function print_scripts()
    {
        wp_enqueue_script('jquery');

        wp_register_script($this->slug().'-lightbox', $this->plugins_url('assets/js/atomicon-gallery.lightbox.js'), array('jquery'));
        wp_enqueue_script($this->slug().'-lightbox');

        wp_register_script($this->slug().'-script', $this->plugins_url('assets/js/atomicon-gallery.script.js'), array('jquery'));
        wp_enqueue_script($this->slug().'-script');
    }

    /**
     * atomicon_gallery_plugin::print_styles()
     *
     * @return
     */
    function print_styles()
    {
    	wp_enqueue_style('atomicon-gallery-lightbox', $this->plugins_url('assets/css/atomicon-gallery.lightbox.css'));
    	if (is_admin())
    	{
        	wp_enqueue_style('atomicon-gallery-admin', $this->plugins_url('assets/css/atomicon-gallery.admin.css'));
    	}
    	else
    	{
    		wp_enqueue_style('atomicon-gallery-admin', $this->plugins_url('assets/css/atomicon-gallery.front-end.css'));
    	}
    }

    /* End of standard wordpress actions */

    /* The admin options page */

    /**
     * atomicon_gallery_plugin::admin_page()
     *
     * @return void
     */
    function admin_page()
    {
    	$folder = $this->atomicon_gallery->get_current_folder();

    	if (!$this->atomicon_gallery->is_gd_installed())
    	{
    		$this->add_message( __('GD Library is not installed! Resizing is unavailable'), true);
    	}

     	if ( ($deleted_items = $this->atomicon_gallery->delete()) > 0 )
     	{
     		$this->add_message( sprintf(__('Deleted %d item(s)'), $deleted_items) );
       		$this->set_session('messages', $this->_messages);
         	$this->refresh();
     	}

        if ($this->atomicon_gallery->create_folder())
        {
        	$this->set_session('messages', $this->_messages);
        	$this->refresh();
        }

        if ($this->atomicon_gallery->upload_files()!==NULL)
        {
         	$this->set_session('messages', $this->_messages);
         	$this->refresh();
        }

        if ($this->save_options())
        {
            $this->add_message('Settings saved', false);
        }

        $this->messages();

		$folders = $this->atomicon_gallery->get_folders();
        $gallery = $this->atomicon_gallery->get_gallery($folder);

		$folders = is_array($folders) ? $folders : array();

		$folders = array(''=>__('/ (root)')) + $folders;

        $data = array('gallery' => $gallery, 'folder' => $folder, 'folders' => $folders);

        $this->load_view('admin', $data);
    }


    function refresh($folder = null)
    {
    	if ($folder===null)
		{
    		$folder = $this->atomicon_gallery->get_current_folder();
    	}
    	$url = $this->admin_url();
    	$url .= empty($folder) ? '' : '&folder='.urlencode($folder);
      	wp_redirect($url);
    }

    /**
     * atomicon_gallery_plugin::plugins_url()
     *
     * @param string $path
     * @param string $plugin
     * @return url to the plugin (if no plugin is given.. the current file(this) will be used)
     */
    function plugins_url($path = '', $plugin = null)
    {
        $plugin = empty($plugin) ? __FILE__ : $plugin;
        return plugins_url($path, $plugin);
    }

    /**
     * atomicon_gallery_plugin::plugins_path()
     *
     * @param string $path
     * @param string $plugin
     * @return path to the plugin (if no plugin is given.. the current file(this) will be used)
     */
    function plugins_path($path = '', $plugin = null)
    {
        $result = '';

        if (empty($plugin))
        {
            $result = dirname(__FILE__) . DIRECTORY_SEPARATOR . $path;
        }
        else
        {
            $result = dirname(__FILE__) . DIRECTORY_SEPARATOR . $path;
            $pos = strrpos($result, 'plugins');
            if ($pos !== false)
            {
                $result = substr($result, 0, $pos + strlen('plugins')) . $path;
            }
        }
        return $result;
    }

    /**
     * atomicon_gallery_plugin::load_model()
     *
     * @param string $model (model name without extension located in the models directory)
     * @param string $variable_name (how to declare it in this class)
     * @return true if the model is loaded and false if the model does not exist
     */
    function load_model($model, $variable_name = null)
    {
        if (isset($this->$model))
        {
            return true;
        }

        $path = dirname(__FILE__) . '/models/' . $model . '.php';
        if (!file_exists($path))
        {
            return false;
        }

        $variable_name = empty($variable_name) ? $model : $variable_name;

        require_once ($path);
        $this->$variable_name = new $model;
        $this->$variable_name->plugin = $this;

        return true;
    }

    /**
     * atomicon_gallery_plugin::load_view()
     *
     * @param string $view (name of the view located in the views directory without .php extenstion)
     * @param array $data (an array of assoc items, these items will be transformed to variables)
     * @param bool $return
     * @return returns boolean (if $return==false)) or the view as string
     */
    function load_view($view, $data = array(), $return = false)
    {
        $path = dirname(__FILE__) . '/views/' . $view . '.php';
        if (!file_exists($path))
        {
            return false;
        }

        if (is_array($data))
        {
            extract($data);
        }

        ob_start();
        include ($path);
        $result = ob_get_contents();
        ob_end_clean();

        if ($return)
        {
            return $result;
        }

        echo $result;
        return true;
    }

    /**
     * atomicon_gallery_plugin::partial()
     *
     * @param string $partial
     * @param string $data
     * @param boolean $return
     * @return The same a load_view only the subdir 'partials' is prepended.
     */
    function partial($partial, $data = array(), $return = false)
    {
        return $this->load_view('partials/' . $partial, $data, $return);
    }

    /**
     * atomicon_gallery_plugin::title()
     *
     * @return string The title of the plugin
     */
    function title()
    {
        return $this->_title;
    }

    /**
     * atomicon_gallery_plugin::slug()
     *
     * @param string $value
     * @return string A slugged value of the string (if no value is given the plugin title will be used)
     */
    function slug($value = null)
    {
    	$value = empty($value) ? $this->title() : $value;
        if (!empty($this->_slug))
        {
            $value = $this->_slug;
        }
        else
        {
            $value = preg_replace("/[^a-zA-Z0-9-]/", "-", $value);
            while (strpos($value, '--'))
            {
                $value = str_replace('--', '-', $value);
            }
        }
        return strtolower($value);
    }

    /**
     * atomicon_gallery_plugin::admin_url()
     *
     * @returns the current admin url
     */
    function admin_url()
    {
        $php_self = isset($_SERVER['PHP_SELF']) ? $_SERVER['PHP_SELF'] : '';
        $page = isset($_GET['page']) ? $_GET['page'] : $this->slug();
        return $php_self . '?page=' . $page;
    }

    /**
     * atomicon_gallery_plugin::get_option()
     *
     * @param string $name Name of the option
     * @param mixed $default Default value if the option does not exist
     * @param bool $check_post Whether to check the post values
     * @return mixed Option value
     */
    function get_option($name, $default = false, $check_post = true)
    {
        if ($check_post)
        {
            if (($post = $this->get_post()) !== false)
            {
                if (isset($post[$name]))
                {
                    return $post[$name];
                }
            }
        }
        return isset($this->_options[$name]) ? $this->_options[$name] : $default;
    }

    /**
     * atomicon_gallery_plugin::save_options()
     *
     * @return bool
     */
    function save_options()
    {
        if (($post = $this->get_post()) !== false)
        {
            update_option($this->slug(), $post);
            return true;
        }
        return false;
    }

    /**
     * atomicon_gallery_plugin::get_post()
     *
     * @return an array with post values or false if it is not found.
     */
    function get_post()
    {
        if ($_POST && isset($_POST[$this->slug()]) && is_array($_POST[$this->slug()]))
        {
            return $_POST[$this->slug()];
        }
        return false;
    }

    /**
     * atomicon_gallery_plugin::message()
     *
     * @param string $message Message to display
     * @param bool $error Whether it is an error or an updated message
     * @param bool $return Return the HTML or print it directly to the screen
     * @return string
     */
    function message($message, $error = false, $return = false)
    {
        $html = '<div class="' . ($error ? 'error' : 'updated') . '"><p>' . $message . '</p></div>' . "\n";
        if ($return)
        {
            return $html;
        }
        echo $html;
    }

    /**
     * atomicon_gallery_plugin::add_message()
     *
     * @param string $message Message to display
     * @param bool $error Whether it is an error or an updated message
     * @return void
     */
    function add_message($message, $error = false)
    {
        $key = $error ? 'error' : 'updated';
        $this->_messages[$key][] = $message;
    }

    /**
     * atomicon_gallery_plugin::messages()
     *
     * @param bool $return Return the HTML or print it directly to the screen
     * @return if $return==true it returns HTML else void
     */
    function messages($return = false)
    {
        $result = '';
        $error = '';
        $updated = '';

        foreach ($this->_messages['error'] as $message)
        {
            $error .= $message . '<br>';
        }

        foreach ($this->_messages['updated'] as $message)
        {
            $updated .= $message . '<br>';
        }

        if ($error != '')
        {
            $result .= $this->message($error, true, true);
        }

        if ($updated != '')
        {
            $result .= $this->message($updated, false, true);
        }

        if ($return)
        {
            return $result;
        }
        echo $result;
    }

    function get_session($name, $default = FALSE)
    {
    	$result = $default;

    	$name = $this->slug() . '-' . $name;

    	if (!session_id()) {
        	@session_start();
        }

    	if (session_id())
		{
			if (isset($_SESSION) && is_array($_SESSION) && isset($_SESSION[$name]))
			{
				$result = $_SESSION[$name];
			}
    	}
    	return $result;
    }
    function set_session($name, $value)
    {
    	$result = false;

    	$name = $this->slug() . '-' . $name;

    	if (!session_id()) {
        	@session_start();
        }

    	if (session_id())
		{
			if (isset($_SESSION) && is_array($_SESSION))
			{
				$_SESSION[$name] = $value;
				$result = true;
			}
    	}
    	return $result;
    }
    function unset_session($name)
    {
    	$result = false;

    	$name = $this->slug() . '-' . $name;

    	if (!session_id()) {
        	@session_start();
        }

    	if (session_id())
		{
			if (isset($_SESSION) && is_array($_SESSION) && isset($_SESSION[$name]))
			{
				unset($_SESSION[$name]);
				$result = true;
			}
    	}
    	return $result;
    }
}

$atomicon_gallery_plugin = new atomicon_gallery_plugin('Atomicon Gallery');