<?php

class atomicon_gallery
{
    public $plugin = null;

    function init_images_folder()
    {
    	$images_folder = $this->get_images_folder('');
    	$images_folder = rtrim($images_folder, '\\/');
    	if (!file_exists($images_folder))
    	{
    		@mkdir($images_folder);
   		}
   		@chmod($images_folder, '0666');
    }

    function upload_files($folder = null)
    {
        $count = null;

        $path = $this->get_images_folder($folder);

        if ($this->plugin && isset($_FILES) && is_array($_FILES) && count($_FILES) > 0)
        {
            foreach ($_FILES as $file)
            {
                $name = $file['name'];
                $size = $file['size'];
                $type = $file['type'];
                $error = $file['error'];
                $tmp_name = $file['tmp_name'];

                if ($size > 0 && $error == UPLOAD_ERR_OK && is_uploaded_file($tmp_name))
                {
                    if ($this->is_zip_file($tmp_name))
                    {
                        $count += $this->unzip_files($tmp_name, $path);
                    }
                    else
                    {
                        //try to read image dimensions
                        $size = @getimagesize($tmp_name);
                        if (!empty($size) && $size[0] > 0 && $size[1] > 0)
                        {
                            $src = $tmp_name;
                            $dest = $path . $name;
                            $ok = @move_uploaded_file($src, $dest);

                            $count = $count === null ? 0 : $count;
                            $count += $ok ? 1 : 0;
                        }
                        else
                        {
                            $this->plugin->add_message(sprintf(__('%s is an invalid image'), $name));
                        }
                    }
                }
            }
            if ($count > 0)
            {
                $this->plugin->add_message(sprintf(__('Uploaded %d image(s)'), $count));
            }
        }
        return $count;
    }

    function is_zip_file($zipfile)
    {
        $result = false;
        if ($this->is_zip_installed())
        {
            $zip = zip_open($zipfile);
            if (is_resource($zip))
            {
                $result = true;
                zip_close($zip);
            }
        }
        return $result;
    }

    function unzip_files($zipfile, $path)
    {
        $count = 0;
        if ($this->is_zip_installed())
        {
            $zip = zip_open($zipfile);
            if (is_resource($zip))
            {
                while ($zip_entry = zip_read($zip))
                {
                	$zip_size = zip_entry_filesize($zip_entry);
                	if ($zip_size>0)
                	{
                		$filename = zip_entry_name($zip_entry);
                		if (($pos=strrpos($filename,'/'))!==FALSE)
                		{
                   			$filename = substr($filename, $pos+1);
                		}

                		if (preg_match('/\.(jpg|jpeg|gif|png)$/i', $filename))
                		{
		                    $fp = @fopen( strtolower($path . $filename), "w");
		                    if (is_resource($fp))
		                    {
		                        if (zip_entry_open($zip, $zip_entry, "r"))
		                        {
		                            $buffer = zip_entry_read($zip_entry, $zip_size);
		                            fwrite($fp, $buffer);
		                            zip_entry_close($zip_entry);
		                            $count++;
		                        }
		                        fclose($fp);
		                    }
                  		}
                 	}
                }
                zip_close($zip);
            }
        }
        return $count;
    }

    function get_folders($path = null, $result = array())
    {
    	$base_path = $this->get_images_folder('');
    	$path = $path === null ? $base_path : $path;

		if (file_exists($path))
		{
	     	if ($handle = opendir($path))
			{
			    while (false !== ($folder = readdir($handle)))
				{
					if ($folder[0]!='.' && is_dir($path.$folder))
					{
						$value = str_replace($base_path, '', $path.$folder);
						$value = str_replace('\\', '/', $value);
						$result[$value] = $value;
						$result += $this->get_folders($path.$folder.'/', $result);
			        }
			    }
			    closedir($handle);
			}
		}
		asort($result);
		return $result;
    }

    function get_gallery($folder = '')
    {
        $result = array('folders' => array(), 'files' => array(), );

        $images_folder_base = $this->get_images_folder('');
        $images_folder = $this->get_images_folder($folder);
        $images_url = $this->get_images_url($folder);
        $folder_up = null;

        if ($folder != '')
        {
            if (($pos = strrpos($folder, '/')) !== false)
            {
                $folder_up = substr($folder, 0, $pos);
            }
            else
            {
                $folder_up = '';
            }
        }

        if ($images_folder != '' && file_exists($images_folder))
        {
            if ($handle = opendir($images_folder))
            {
                while (false !== ($file = readdir($handle)))
                {
                    $filefolder = $images_folder . $file;

                    if ($folder_up !== null)
                    {
                        if (is_admin())
                        {
                            $url = $this->plugin->admin_url() . '&folder=' . urlencode($folder_up);
                        }
                        else
                        {
                            $url = '';
                        }

                        $result['folders'][] = array('up' => true, 'type' => 'folder', 'name' => __('Up'), 'folder' => $filefolder, 'url' => $url, );
                        $folder_up = null;
                    }

                    if (is_dir($filefolder))
                    {
                        if ($file[0] == '.')
                        {
                            continue;
                        }

                        if (is_admin())
                        {
                            $url = $this->plugin->admin_url() . '&folder=' . str_replace('\\', '/', str_replace($images_folder_base, '', $images_folder . $file));
                        }
                        else
                        {
                            $url = '';
                        }

                        $result['folders'][] = array('up' => false, 'type' => 'folder', 'name' => $file, 'folder' => $filefolder, 'url' => $url, );
                    }
                    else
                        if (is_file($filefolder) && strpos($file, 'ag-tn-')!==0)
                        {
                            $size = @getimagesize($filefolder);
                            if (!empty($size) && $size[0] > 0 && $size[1] > 0)
                            {
                            	$thumbnail = $images_folder . 'ag-tn-'. $file;
                            	$thumbnail_dimensions = '';

                            	$thumbnail_exists = file_exists($thumbnail);

								if ($thumbnail_exists)
								{
	                            	$thumbnail_dimensions = @getimagesize($thumbnail);
			                        if (!empty($thumbnail_dimensions) && $thumbnail_dimensions[0] > 0 && $thumbnail_dimensions[1] > 0)
			                        {
										$thumbnail_dimensions = $thumbnail_dimensions[0] . ' x ' . $thumbnail_dimensions[1];
		                        	}
		                        	else
		                        	{
		                        		$thumbnail_exists = false;
		                        		$thumbnail_dimensions = '';
		                        	}
         						}

                            	$thumbnail = $thumbnail_exists ? ($images_url.'ag-tn-'.$file) : ($images_url.$file);

                                $result['files'][] = array(
									'type' => 'file',
									'name' => $file,
									'folder' => $filefolder,
									'width' => $size[0],
									'height' => $size[1],
									'dimensions' => $size[0] . ' x ' . $size[1],
									'url' => $images_url . $file,
									'thumbnail' => $thumbnail,
									'thumbnail_dimensions' => $thumbnail_dimensions,
									'thumbnail_exists' => $thumbnail_exists,
								);
                            }
                        }
                }
                closedir($handle);
            } // handle
        }

        usort($result['folders'], array($this, '_sort_folders'));
        usort($result['files'], array($this, '_sort_files'));

        return $result;
    }

	function delete_item($path)
	{
		$count = 0;

		if (!file_exists($path))
		{
			return $count;
		}

		if (is_dir($path))
		{
			$objects = scandir($path);
			foreach ($objects as $object)
			{
				$object = (string)$object;
				if ($object != '' && $object[0]!='.')
				{
					$count += $this->delete_item($path.'/'.$object);
				}
			}
			reset($objects);
			$result = @rmdir($path);
			$count += $result;
		}
		else
		{
			$result = @unlink($path);
			$count += $result;
		}
		return $count;
	}

    function delete()
    {
    	$count = 0;
    	$base  = $this->get_images_folder(null);
		$items = isset($_POST['atomicon-gallery-items']) ? $_POST['atomicon-gallery-items'] : array();
    	$items = is_array($items) ? $items : array();

		foreach($items as $item) :
			$count += $this->delete_item($base . $item);
			$this->delete_item($base . 'ag-tn-' . $item); //try to delete thumbnail (don't add to the count since they are invisible)
		endforeach;

		return $count;
    }

    function _sort_folders($a, $b)
    {
        if ($a['up'])
        {
            return - 1;
        }
        if ($b['up'])
        {
            return 1;
        }

        return strcasecmp($a['name'], $b['name']);
    }

    function _sort_files($a, $b)
    {
        return strcasecmp($a['name'], $b['name']);
    }

    function create_folder($folder_name = null)
    {
        if ($folder_name === null)
        {
            $folder_name = isset($_REQUEST['atomicon-gallery-create-folder']) ? $_REQUEST['atomicon-gallery-create-folder'] : '';
        }

        $folder_name = trim((string )$folder_name);
        if ($folder_name != '')
        {
            $path = $this->get_images_folder() . $folder_name;
            if (!file_exists($path))
            {
                $ok = mkdir($path);
                if ($ok !== false)
                {
                    $this->plugin->add_message(__('Folder created'), false);
                    return true;
                }
                else
                {
                    $this->plugin->add_message(__('Error creating folder'), true);
                }
            }
            else
            {
                $this->plugin->add_message(__('Folder already exists'), true);
            }
        }
        return false;
    }

    function get_images_folder($folder = null)
    {
        if ($folder === null)
        {
            $folder = isset($_REQUEST['folder']) ? $_REQUEST['folder'] : '';
        }

        if ($folder != '')
        {
            if (!in_array(substr($folder, -1), array('\\', '/')))
            {
                $folder .= '/';
            }
        }

        $result = '';
        if ($this->plugin)
        {
        	$result = WP_CONTENT_DIR .'/atomicon-gallery/'.$folder;
        }
        return $result;
    }

    function get_images_url($folder = null)
    {
        if ($folder != '')
        {
            if (!in_array(substr($folder, -1), array('\\', '/')))
            {
                $folder .= '/';
            }
        }
        $result = WP_CONTENT_URL.'/atomicon-gallery/'.$folder;
        return $result;
    }

    function get_current_folder()
    {
        return isset($_REQUEST['folder']) ? $_REQUEST['folder'] : null;
    }

    function is_zip_installed()
    {
        return function_exists('zip_open') ? true : false;
    }

    function is_gd_installed()
    {
        return (extension_loaded('gd') && function_exists('gd_info')) ? true : false;
    }

    /**
     * atomicon_gallery::gd_resize()
     *
     * @param mixed $maxWidth
     * @param mixed $maxHeight
     * @param mixed $origFile
     * @param mixed $destFile
     * @return false on error (filename on success)
     */
    function gd_resize($maxWidth, $maxHeight, $origFile, $destFile)
    {
        /* check if the file exists*/
        if (!file_exists($origFile))
        {
            return false;
        }

        /* Load image*/
        $img = null;

		$ext = '';
        $pos = strrpos($origFile,'.');
        $ext = $pos!==FALSE ? substr($origFile,$pos+1) : '';
        $ext = strtolower($ext);

        $memory = ini_get('memory_limit');
        $intmemory = rtrim($memory, 'M');

        if ($intmemory < 50)
        {
            ini_set('memory_limit', '50M');
        }
        if ($ext == 'jpg' || $ext == 'jpeg')
        {
            $img = imagecreatefromjpeg($origFile);
            $header = "image/jpeg";
        }
        else
            if ($ext == 'png')
            {
                $img = imagecreatefrompng($origFile);
                $header = "image/png";
                /* Only if your version of GD includes GIF support*/
            }
            else
                if ($ext == 'gif')
                {
                    if (function_exists('imagecreatefromgif'))
                    {
                        $img = imagecreatefromgif($origFile);
                        $header = "image/gif";
                    }
                    else
                    {
                        return false;
                    }
                }
        /* If an image was successfully loaded, test the image for size*/
        if ($img)
        {
            /* Get image size and scale ratio*/
            $width = imagesx($img);
            $height = imagesy($img);

            $scale = min($maxWidth / $width, $maxHeight / $height);

            $new_width = floor($scale * $width);
            $new_height = floor($scale * $height);
            /* Create a new temporary image*/
            $tmp_img = imagecreatetruecolor($new_width, $new_height);
            /* If the image is larger than the max shrink it*/
            if ($scale < 1)
            {
                /* Copy and resize old image into new image*/
                imagecopyresampled($tmp_img, $img, 0, 0, 0, 0, $new_width, $new_height, $width, $height);
                imagedestroy($img);
                $img = $tmp_img;
            }
            else
            {
                if ($origFile != $destFile)
                {
                    @unlink($destFile);
                    @copy($origFile, $destFile);
                    return $destFile;
                }
            }
        }
        if (!$img)
        {
            return false;
        }

        /* save the file */

        if ($header == "image/jpeg")
        {
            ob_start();
            imagejpeg($img, "", 100);
            $image = ob_get_contents();
            ob_end_clean();
            @unlink($destFile);
            file_put_contents($destFile, $image);
        }
        else
        {
            if ($header == "image/png")
            {
                ob_start();
                imagepng($img, "", 0);
                $image = ob_get_contents();
                ob_end_clean();
                @unlink($destFile);
                file_put_contents($destFile, $image);
            }
            else
            {
                if (function_exists("imagegif"))
                {
                    ob_start();
                    imagegif($img, "", 100);
                    $image = ob_get_contents();
                    ob_end_clean();
                    @unlink($destFile);
                    file_put_contents($destFile, $image);
                }
                else
                {
                    /* try using imagemagick to convert gif to png:*/
                    $image_file = imgkConvertImage($image_file, $baseDir, $destDir, ".png");
                }
            }
        }
        ini_set('memory_limit', $memory);
        return $destFile;
    }
}
